package org.gcube.datacatalogue.ckanutillibrary.ckan;

import static com.google.common.base.Preconditions.checkArgument;
import static com.google.common.base.Preconditions.checkNotNull;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import eu.trentorise.opendata.jackan.internal.org.apache.http.HttpResponse;
import eu.trentorise.opendata.jackan.internal.org.apache.http.HttpStatus;
import eu.trentorise.opendata.jackan.internal.org.apache.http.client.methods.HttpPost;
import eu.trentorise.opendata.jackan.internal.org.apache.http.entity.StringEntity;
import eu.trentorise.opendata.jackan.internal.org.apache.http.impl.client.CloseableHttpClient;
import eu.trentorise.opendata.jackan.internal.org.apache.http.impl.client.HttpClientBuilder;

/**
 * The Class DirectCkanCaller.
 *
 * @author Francesco Mangiacrapa at ISTI-CNR Pisa (Italy)
 * Jun 3, 2020
 */
public class DirectCkanCaller {
	
	private static final String PATH_PACKAGE_PATCH = "/api/3/action/package_patch";
	public final static String PATH_SET_PRIVATE_DATASET = "/api/3/action/bulk_update_private";
	public final static String PATH_SET_PUBLIC_DATASET = "/api/3/action/bulk_update_public";
	private String catalogueURL;
	
	private static final Logger LOG = LoggerFactory.getLogger(DirectCkanCaller.class);
	
	
	/**
	 * Instantiates a new direct ckan caller.
	 *
	 * @param catalogueURL the catalogue URL
	 */
	public DirectCkanCaller(String catalogueURL){
		this.catalogueURL = catalogueURL;
	}
	
	/**
	 * Set dataset private/public.
	 *
	 * @param priv the priv
	 * @param organizationId (NOTE: The ID, not the name!)
	 * @param datasetId (NOTE: The ID, not the name!)
	 * @param apiKey the user's api key
	 * @return true on success, false otherwise
	 */
	public boolean setDatasetPrivate(boolean priv, String organizationId,
			String datasetId, String apiKey) {

		// checks
		checkNotNull(organizationId);
		checkNotNull(apiKey);
		checkNotNull(datasetId);
		checkArgument(!apiKey.isEmpty());
		checkArgument(!datasetId.isEmpty());
		checkArgument(!organizationId.isEmpty());

		JSONObject obj = new JSONObject();
		obj.put("org_id", organizationId);

		JSONArray array = new JSONArray();
		array.add(datasetId);
		obj.put("datasets", array);

		try(CloseableHttpClient httpClient = HttpClientBuilder.create().build();) {
			HttpPost request;

			if(priv)
				request = new HttpPost(catalogueURL + PATH_SET_PRIVATE_DATASET);
			else
				request = new HttpPost(catalogueURL + PATH_SET_PUBLIC_DATASET);

			LOG.info("Excuting request for making dataset with id " + datasetId + " " + (priv? "private" : "public"));

			request.addHeader("Authorization", apiKey);
			StringEntity params = new StringEntity(obj.toJSONString());
			request.setEntity(params);
			HttpResponse response = httpClient.execute(request);

			LOG.info("Response code is " + response.getStatusLine().getStatusCode() + " and response message is " + response.getStatusLine().getReasonPhrase());

			if(response.getStatusLine().getStatusCode() == HttpStatus.SC_OK)
				return true;

		}catch (Exception ex) {
			LOG.error("Error while trying to set private the dataset ", ex);
		}


		return false;
	}
	
	/**
	 * Sets the searchable field.
	 *
	 * @param datasetId the dataset id
	 * @param searchable the searchable
	 * @return true, if successful
	 */
	public boolean setSearchableField(String datasetId, boolean searchable, String apiKey) {

		// checks
		checkNotNull(datasetId);
		checkArgument(!datasetId.isEmpty());
		String searchableAsString = searchable ? "True" : "False";

		// Patch package path
		String patchPackage = catalogueURL + PATH_PACKAGE_PATCH;

		JSONObject obj = new JSONObject();
		obj.put("id", datasetId);
		obj.put("searchable", searchableAsString);

		try(CloseableHttpClient httpClient = HttpClientBuilder.create().build();) {
			HttpPost request = new HttpPost(patchPackage);
			request.addHeader("Authorization", apiKey);
			StringEntity params = new StringEntity(obj.toJSONString());
			request.setEntity(params);
			HttpResponse response = httpClient.execute(request);
			LOG.debug("Response code is " + response.getStatusLine().getStatusCode() + " and response message is " + response.getStatusLine().getReasonPhrase());

			if(response.getStatusLine().getStatusCode() == HttpStatus.SC_OK)
				return true;

		}catch (Exception ex) {
			LOG.error("Error while trying to set searchable the dataset ", ex);
		}

		return false;
	}
	

}
